#!/bin/bash

# Common variables used in setting up the service
MODE=$(config-get mode)
BASEDIR=$(config-get basedir)
BZR_SOURCE_BRANCH=$(config-get bzr_branch)
BZR_SOURCE_REVNO=$(config-get bzr_revno)
BZR_THEME_BRANCH=$(config-get theme_branch)
BZR_THEME_REVNO=$(config-get theme_revno)
BZR_SSO_BRANCH=$(config-get sso_auth_branch)
BZR_SSO_REVNO=$(config-get sso_auth_revno)
BASEDIR=$(config-get basedir)
SITENAME=$(config-get sitename)
INSTANCE_TYPE=$(config-get instance_type)
REGION=$(config-get region)
USER_CODE_OWNER=$(config-get user_code_owner)
GROUP_CODE_OWNER=$(config-get group_code_owner)
USER_CODE_RUNNER=$(config-get user_code_runner)
GROUP_CODE_RUNNER=$(config-get group_code_runner)
LISTEN_PORT=$(config-get listen_port)
DEBUG_MODE=$(config-get debug_mode)
ADMIN_NODE=$(config-get admin_node)
WSGI_WORKERS=$(config-get wsgi_workers)
WSGI_WORKER_CLASS=$(config-get wsgi_worker_class)
WSGI_WORKER_CONNECTIONS=$(config-get wsgi_worker_connections)
WSGI_MAX_REQUESTS=$(config-get wsgi_max_requests)
WSGI_BACKLOG=$(config-get wsgi_backlog)
WSGI_TIMEOUT=$(config-get wsgi_timeout)
WSGI_KEEP_ALIVE=$(config-get wsgi_keep_alive)

DB_ROLE=$(config-get db_role)
NRPE_URL_CHECK_PARAMS=$(config-get nrpe_url_check_params)

DEPLOY_ROOT=/srv/${BASEDIR}/${INSTANCE_TYPE}
LOG_DIR=/srv/${BASEDIR}/${INSTANCE_TYPE}-logs
CODE_LOCATION=${DEPLOY_ROOT}/summit-website
DJANGO_PROJECT=${CODE_LOCATION}/summit
MANAGE_PY=${DJANGO_PROJECT}/manage.py
LOCAL_CONFIG_LOCATION=${CODE_LOCATION}/summit

LOCAL_SETTINGS_FILE=${LOCAL_CONFIG_LOCATION}/local_settings.py
DB_SETTINGS_FILE=${DEPLOY_ROOT}/db_settings.py

# And this is for generating the deploymgr script from a template
JUJU_CONFIGS=$(/bin/mktemp --suffix=.juju)
config-get --format=json -o $JUJU_CONFIGS

function update_code_branches {
    # If this is a new revision of summit get the new code
    if [ ! -d ${CODE_LOCATION}-${BZR_SOURCE_REVNO} ]; then
        juju-log "Branching from ${BZR_SOURCE_BRANCH}:${BZR_SOURCE_REVNO} to ${CODE_LOCATION}-${BZR_SOURCE_REVNO}"
        bzr branch -r ${BZR_SOURCE_REVNO} ${BZR_SOURCE_BRANCH} ${CODE_LOCATION}-${BZR_SOURCE_REVNO}
    fi

    # If we have an active branch symlink already, delete it
    if [ -d ${CODE_LOCATION} ]; then
        rm ${CODE_LOCATION}
    fi

    juju-log "Creating symlink from ${CODE_LOCATION}-${BZR_SOURCE_REVNO} to ${CODE_LOCATION}"
    ln -s ${CODE_LOCATION}-${BZR_SOURCE_REVNO} ${CODE_LOCATION}

    # If we have db_settings.py, link it to the new branch
    if [[ -f ${DB_SETTINGS_FILE} && ! -f ${LOCAL_CONFIG_LOCATION}/db_settings.py ]]; then
        juju-log "Creating symlink from${DB_SETTINGS_FILE} to ${LOCAL_CONFIG_LOCATION}/db_settings.py"
        ln -s ${DB_SETTINGS_FILE} ${LOCAL_CONFIG_LOCATION}/db_settings.py
    fi

    # If there is no theme checkout in the summit-website branch get it now
    if [ ! -d ${DEPLOY_ROOT}/ubuntu_website-${BZR_THEME_REVNO} ]; then
        juju-log "Branching Theme from ${BZR_THEME_BRANCH}:${BZR_THEME_REVNO} to ${DEPLOY_ROOT}/ubuntu_website-${BZR_THEME_REVNO}"
        bzr branch -r ${BZR_THEME_REVNO} ${BZR_THEME_BRANCH} ${DEPLOY_ROOT}/ubuntu_website-${BZR_THEME_REVNO}
    fi

    if [ -d ${CODE_LOCATION}/summit/ubuntu_website ]; then
        rm -r ${CODE_LOCATION}/summit/ubuntu_website
    fi
    juju-log "Creating symlink from ${DEPLOY_ROOT}/ubuntu_website-${BZR_THEME_REVNO} to ${CODE_LOCATION}/summit/ubuntu_website"
    ln -s ${DEPLOY_ROOT}/ubuntu_website-${BZR_THEME_REVNO} ${CODE_LOCATION}/summit/ubuntu_website

    # If there is no django_openid_auth checkout in the summit-website branch get it now
    if [ ! -d ${DEPLOY_ROOT}/django_openid_auth-${BZR_SSO_REVNO} ]; then
        juju-log "Branching from ${BZR_SSO_BRANCH}:${BZR_SSO_REVNO} to ${DEPLOY_ROOT}/django_openid_auth-${BZR_SSO_REVNO}"
        bzr branch -r ${BZR_SSO_REVNO} ${BZR_SSO_BRANCH} ${DEPLOY_ROOT}/django_openid_auth-${BZR_SSO_REVNO}
    fi

    if [ -d ${CODE_LOCATION}-${BZR_SOURCE_REVNO}/summit/django_openid_auth ]; then
        rm -r ${CODE_LOCATION}-${BZR_SOURCE_REVNO}/summit/django_openid_auth
    fi
    juju-log "Creating symlink from ${DEPLOY_ROOT}/django_openid_auth-${BZR_SSO_REVNO}/django_openid_auth to ${CODE_LOCATION}-${BZR_SOURCE_REVNO}/summit/django_openid_auth"
    ln -s ${DEPLOY_ROOT}/django_openid_auth-${BZR_SSO_REVNO}/django_openid_auth ${CODE_LOCATION}-${BZR_SOURCE_REVNO}/summit/django_openid_auth
}

function manage_db {
    juju-log "Setting db perms"
    PYTHONPATH=${DEPLOY_ROOT} ./hooks/grant_perms.py ${DB_ROLE} || exit 1
    juju-log "Syncing the database for summit-website..."
    python ${MANAGE_PY} syncdb --noinput --settings=local_settings
    python ${MANAGE_PY} migrate --noinput --settings=local_settings
    python ${MANAGE_PY} init-summit --noinput --settings=local_settings
}

function update_local_settings {
    juju-log "Creating local settings in ${LOCAL_CONFIG_LOCATION}/local_settings.py"
    SECRET_KEY=$(pwgen -s 50 1)
    cat > ${LOCAL_SETTINGS_FILE} <<EOF
# Regularly update Summit

from settings import *

DEBUG=${DEBUG_MODE}

SECRET_KEY='${SECRET_KEY}'

# Database configs
try:
    from db_settings import *
except Exception:
    pass

# Theme configs
import ubuntu_website

INSTALLED_APPS.append('ubuntu_website')

TEMPLATE_CONTEXT_PROCESSORS += (
    "ubuntu_website.media_processor",
    "ubuntu_website.popup_check",
)
TEMPLATE_DIRS += (
    ubuntu_website.TEMPLATE_DIR,
)

THEME_MEDIA = ubuntu_website.MEDIA_ROOT

    
# Cumulus configs

EOF
}

function update_cron {
    juju-log "Creating cron job in /etc/cron.d/summit-website"
    CRON_SCRIPT=/etc/cron.d/summit-website
    CRON_TARGET=/etc/cron.hourly/summit-website
    cat > ${CRON_SCRIPT} <<EOF
# Local instance overrides

cd ${DJANGO_PROJECT};
export DJANGO_SETTINGS=local_settings
python manage.py lpupdate >> ${DEPLOY_ROOT}-logs/lpupdate.log 2>&1
python manage.py trackfix >> ${DEPLOY_ROOT}-logs/trackfix.log 2>&1

EOF

    if [ -e ${CRON_TARGET} ]; then
        rm ${CRON_TARGET}
    fi
    juju-log "Creating symlink from ${CRON_SCRIPT} to ${CRON_TARGET}"
    ln -s ${CRON_SCRIPT} ${CRON_TARGET}
}

function update_static_files {
    juju-log "Collecting static files..."
    python ${MANAGE_PY} collectstatic --noinput --settings=local_settings
}    
function ctrl_service {
    # Check if there is an upstart or sysvinit service defined and issue the
    # requested command if there is. This is used to control services in a
    # friendly way when errexit is on.
    service_name=$1
    service_cmd=$2
    ( service --status-all 2>1 | grep -w $service_name ) && service $service_name $service_cmd
    ( initctl list 2>1 | grep -w $service_name ) && service $service_name $service_cmd
    return 0
}
